import enum


class Properties(enum.IntEnum):  # pragma: no cover
    keyboard_accelerator = 2
    alerts = 3
    justification = 4
    button_1_label = 5
    button_2_label = 6
    button_3_label = 7
    default_alert_button = 8
    message = 9
    alert_style = 10
    attached_libraries = 11
    automatic_select = 12
    automatic_display = 13
    automatic_query = 14
    automatic_refresh = 15
    automatic_skip = 16
    background_color = 17
    database_item = 18
    bevel = 19
    mirror_item_object = 20
    data_blocks = 21
    bottom_title = 22
    graphics = 23
    canvases = 24
    canvas = 25
    canvas_object_pointer = 26
    case_info = 27
    case_insensitive_query = 28
    case_restriction = 29
    check_box_mapping_of_other_values = 30
    value_when_checked = 33
    close_allowed = 34
    canvas_type = 35
    synchronize_with_item = 36
    column_data_type = 37
    column_mapping_object = 38
    enforce_column_security = 39
    record_group_colspecs = 40
    command_text = 41
    command_type = 42
    comments = 43
    image_format = 44
    console_window = 45
    image_depth = 46
    cursor_mode = 48
    deferred = 49
    delete_allowed = 50
    delete_record_behavior = 51
    prompt_foreground_color = 52
    detail_data_block = 53
    default_button = 54
    initial_keyboard_state = 55
    direction = 57
    visible = 58
    display_without_privilege = 59
    display_viewport = 60
    display_width = 61
    viewport_x_position = 62
    viewport_y_position = 63
    editors = 64
    editor = 65
    editor_object_pointer = 66
    editor_x_position = 67
    editor_y_position = 68
    enabled = 69
    execution_hierarchy = 70
    fill_pattern = 73
    fire_in_enter_query_mode = 74
    first_navigation_data_block = 75
    first_data_block_object = 76
    resize_allowed = 78
    menu_module = 79
    menu_role = 80
    initial_menu = 81
    font_name = 82
    font_size = 83
    font_style = 84
    font_weight = 85
    font_spacing = 86
    foreground_color = 87
    format_mask = 88
    form_parameters = 89
    parameter_data_type = 90
    parameter_initial_value = 91
    filename = 92
    execution_mode = 93
    height = 94
    highest_allowed_value = 95
    hint = 96
    show_horizontal_scroll_bar = 97
    horizontal_toolbar_canvas = 98
    iconic = 99
    minimize_allowed = 100
    icon_filename = 101
    minimized_title = 102
    inherit_menu = 103
    insert_allowed = 104
    items = 105
    item_type = 106
    copy_value_from_item = 107
    data_type = 108
    initial_value = 109
    join_condition = 115
    keep_cursor_position = 116
    key_mode = 117
    label = 118
    prompt_background_color = 119
    pl_sql_library_location = 120
    pl_sql_library_source = 121
    locking_mode = 123
    lock_record = 124
    filter_before_display = 125
    lovs = 126
    list_of_values = 127
    lov_object_pointer = 128
    list_type = 129
    validate_from_list = 130
    list_x_position = 131
    list_y_position = 132
    lowest_allowed_value = 133
    list_style = 134
    magic_item = 135
    main_menu = 136
    maximum_length = 137
    menus = 138
    form_horizontal_toolbar_canvas = 139
    form_vertical_toolbar_canvas = 140
    menu_directory = 141
    menu_filename = 142
    menu_item_object = 143
    menu_item_type = 144
    compression_quality = 145
    associated_menus = 146
    menu_item_radio_group = 147
    access_key = 148
    modal = 149
    mouse_navigation_limit = 150
    mouse_navigate = 151
    move_allowed = 152
    multi_line = 153
    name_ = 154  # fixme: conflicts with enum.IntEnum...
    prompt_fill_pattern = 155
    keyboard_navigable = 156
    navigation_style = 157
    next_object = 159
    next_navigation_data_block = 160
    next_data_block_object = 161
    next_navigation_item = 162
    next_item_object = 163
    object_groups = 164
    object_group_child_object = 165
    old_lov_text = 166
    optimizer_hint = 175
    order_by_clause = 176
    mapping_of_other_values = 177
    primary_key = 178
    program_units = 179
    property_classes = 180
    prevent_masterless_operations = 181
    previous_navigation_data_block = 182
    previous_data_block_object = 183
    previous_navigation_item = 184
    previous_item_object = 185
    query_allowed = 186
    query_length = 187
    query_only = 188
    display_quality = 189
    radio_button_object = 190
    raise_on_entry = 191
    radio_button_value = 192
    reading_order = 193
    number_of_records_buffered = 195
    number_of_records_displayed = 196
    query_array_size = 197
    distance_between_records = 198
    record_groups = 199
    record_group = 200
    record_group_object_pointer = 201
    record_group_query = 202
    record_group_type = 203
    record_orientation = 204
    current_record_visual_attribute_group = 205
    relation_object = 206
    hide_on_exit = 207
    rendered = 208
    required = 209
    return_item = 210
    reverse_direction = 211
    hide_object = 213
    prompt_font_name = 214
    savepoint_mode = 215
    scroll_bar_canvas = 216
    scroll_bar_orientation = 217
    scroll_bar_width = 218
    scroll_bar_x_position = 219
    scroll_bar_y_position = 220
    show_scroll_bar = 221
    conceal_data = 222
    display_in_keyboard_help = 223
    sizing_style = 224
    startup_code = 225
    sub_title = 226
    tear_off_menu = 229
    title = 230
    prompt_font_size = 231
    triggers = 232
    keyboard_help_text = 233
    trigger_step_object = 234
    trigger_style = 236
    trigger_text = 237
    value_when_unchecked = 238
    update_allowed = 239
    update_changed_columns_only = 240
    update_only_if_null = 241
    use_3d_controls = 242
    use_security = 243
    visual_attribute_group = 245
    va_object = 246
    prompt_font_style = 247
    calculation_mode = 248
    prompt_font_weight = 249
    prompt_font_spacing = 250
    column_name = 251
    prompt_display_style = 252
    show_vertical_scroll_bar = 253
    viewport_height = 254
    viewport_width = 255
    viewport_x_position_on_canvas = 256
    viewport_y_position_on_canvas = 257
    visual_attributes = 258
    visual_stat = 259
    validation_unit = 260
    vertical_toolbar_canvas = 261
    where_clause = 262
    width = 263
    windows = 264
    enforce_primary_key = 265
    window = 268
    window_object_pointer = 269
    window_style = 270
    primary_canvas = 271
    wrap_style = 272
    x_position = 273
    y_position = 274
    maximize_allowed = 275
    display_hint_automatically = 277
    tooltip = 279
    current_record_va_pointer = 280
    graphics_type = 281
    closed = 282
    arrow_style = 283
    x_corner_radius = 284
    y_corner_radius = 285
    start_angle = 286
    end_angle = 287
    prompt = 288
    prompt_reading_order = 289
    prompt_justification = 290
    previous_object = 291
    prompt_attachment_edge = 292
    prompt_attachment_offset = 293
    prompt_alignment = 294
    prompt_alignment_offset = 295
    owning_object = 296
    dither = 297
    clip_x_position = 298
    clip_y_position = 299
    clip_width = 300
    clip_height = 301
    points = 302
    frame_title_reading_order = 303
    frame_title_alignment = 304
    frame_title_offset = 305
    frame_title_spacing = 306
    frame_title = 307
    layout_style = 308
    frame_alignment = 309
    single_object_alignment = 310
    vertical_fill = 311
    maximum_objects_per_line = 312
    allow_expansion = 313
    horizontal_margin = 314
    vertical_margin = 315
    vertical_object_offset = 316
    horizontal_object_offset = 317
    start_prompt_offset = 318
    top_prompt_offset = 319
    start_prompt_alignment = 320
    top_prompt_alignment = 321
    allow_multi_line_prompts = 322
    allow_top_attached_prompts = 323
    allow_start_attached_prompts = 324
    update_layout = 325
    line_width = 326
    rotation_angle = 327
    edge_foreground_color = 328
    edge_background_color = 329
    edge_pattern = 330
    dash_style = 331
    cap_style = 332
    join_style = 333
    line_spacing = 334
    custom_spacing = 335
    fixed_bounding_box = 336
    wrap_text = 337
    bounding_box_scaleable = 338
    font_scaleable = 339
    horizontal_origin = 340
    vertical_origin = 341
    horizontal_justification = 342
    vertical_justification = 343
    compound_text_object = 344
    formula = 346
    summary_function = 347
    summarized_item = 348
    number_of_items_displayed = 349
    scrollbar_canvas_object_pointer = 350
    argument_mode = 351
    tab_pages = 356
    tab_page = 357
    tab_page_name = 358
    communication_mode = 359
    data_source_data_block = 360
    data_source_x_axis = 361
    data_source_y_axis = 362
    update_on_commit = 363
    update_on_query = 364
    query_name = 365
    popup_menu = 366
    popup_menu_object = 367
    dml_data_target_name = 370
    query_data_source_name = 371
    data_source_column_name = 372
    data_source_column_type = 373
    argument_name = 374
    argument_type = 375
    argument_value = 376
    database_data_block = 377
    query_data_source_type = 378
    query_all_records = 379
    dml_data_target_type = 380
    dml_array_size = 381
    tooltip_visual_attribute_group = 382
    tooltip_va_object = 383
    insert_procedure_name = 419
    update_procedure_name = 422
    delete_procedure_name = 425
    lock_procedure_name = 428
    single_record = 431
    trigger_internal_type = 432
    length = 433
    mandatory = 434
    precision = 435
    scale = 436
    prompt_va_object = 437
    prompt_visual_attribute_group = 438
    layout_data_block = 439
    owning_module = 440
    source_object = 441
    scroll_bar_tab_page = 442
    scrollbar_tab_object = 443
    visible_in_vertical_menu_toolbar = 444
    visible_in_horizontal_menu_toolbar = 445
    tab_attachment_edge = 446
    argument_type_name = 447
    visible_in_menu = 448
    visual_attribute_type = 450
    program_unit_text = 451
    icon_in_menu = 452
    automatic_position = 453
    automatic_column_width = 454
    record_group_fetch_size = 455
    title_va_object = 456
    frame_title_visual_attribute_group = 457
    frame_title_foreground_color = 458
    frame_title_background_color = 459
    frame_title_fill_pattern = 460
    frame_title_font_name = 461
    frame_title_font_size = 462
    frame_title_font_style = 463
    frame_title_font_weight = 464
    frame_title_font_spacing = 465
    corner_style = 466
    interaction_mode = 467
    maximum_query_time = 468
    maximum_records_fetched = 469
    isolation_mode = 470
    count_of_data_items_in_the_column = 471
    real_object_pointed_to_by_the_object_group_child = 472
    reports = 473
    count_of_list_element_items = 474
    precompute_summaries = 475
    report_destination_type = 476
    report_destination_name = 477
    report_destination_format = 478
    report_server = 479
    other_reports_parameters = 480
    share_library_with_form = 487
    keyboard_state = 488
    summarized_block = 490
    scroll_bar_alignment = 491
    shrinkwrap = 492
    graphic_object_font_name = 493
    graphic_object_font_size = 494
    graphic_object_font_style = 495
    graphic_object_text = 496
    parent_objects_module = 497
    parent_objects_module_type = 498
    parent_object_name = 499
    parent_objects_file_name = 500
    parent_objects_file_path = 501
    count_of_roles = 503
    program_unit_type = 504
    submenu_name = 505
    submenu_object = 506
    menu_item_code = 507
    query_column_object = 508
    query_argument_object = 509
    insert_column_object = 510
    insert_argument_object = 511
    update_column_object = 512
    update_argument_object = 513
    delete_column_object = 514
    delete_argument_object = 515
    lock_column_object = 516
    lock_argument_object = 517
    runtime_compatibility_mode = 532
    object_group_type = 533
    graphic_font_weight = 535
    graphic_font_spacing = 536
    parent_objects_first_level_owners_name = 537
    parent_objects_first_level_owners_type = 538
    parent_objects_second_level_owners_name = 539
    parent_objects_second_level_owners_type = 540
    parent_objects_type = 541
    scroll_bar_length = 542
    relation_type = 543
    detail_reference_item = 544
    column_type_name = 545
    parent_column = 546
    alias = 547
    include_ref_item = 548
    help_book_title = 550
    help_book_topic = 551
    allow_empty_branches = 552
    multi_selection = 553
    show_lines = 554
    show_symbols = 555
    tre_record_group = 556
    data_query = 557
    implementation_class = 558
    dml_returning_value = 559
    persistent_client_info_storage = 560
    persistent_client_info_storage_length = 561
    active_style = 562
    width_style = 563
    column_value = 570
    defer_required_enforcement = 572
    data_length_semantics = 573
    event = 574
    event_type = 575
    subscription_name = 576
    scope = 577
    auto_subscribe = 578
    event_implementation_class = 579
    correlation_id = 580
    priority_mode = 581
    view_mode = 582
    report_object_type = 583
    delivery_type = 584
    ssl_connection = 585
    service_location = 586
    report_path = 587
    report_locale = 588
    report_format = 589
    template_name = 590
    report_parameters = 591
    ftp_server_name = 592
    ftp_user = 593
    ftp_file_name = 594
    secure_ftp = 595
    number = 596
    fax_server_name = 597
    to_email = 598
    from_email = 599
    cc_email = 600
    bcc_email = 601
    reply_to = 602
    subject = 603
    body = 604
    mail_server_name = 605
    local_file_name = 606
    number_of_copies = 607
    orientation = 608
    page_range = 609
    side = 610
    tray = 611
    print_server_name = 612
    webdav_auth_type = 613
    webdav_user = 614
    webdav_file_path = 615
    webdav_server_name = 616
    row_banding_frequency = 617
    cursor_style = 618
    gradient_start_side = 619
